/*

--_-_-_-_---
   -_-_-_
    -_-_-
     -__-
    _-_
   _-
   -_
    _-_
    
CyClone Weather Station (Client)

Author: NLD

Date: 2023/Apr/01

Version: 0.2

This code is for the Arduino ProMicro.

Regardless that the code will compile and was tested on:
-Arduino Uno
-Arduino Leonardo

The memory is quite maxed out, in production will work just fine however with debug 
enabled you will get: Low memory available, stability problems may occur.

v0.2:
 -Public release

v0.1:
 -DHT replaced with the modern DHTLib version
 -Liquidcrystal replaced by the I2C version
 -Initial release based on the Natalia Weather Station + Silverlight's code
..............................................................*/

#include <Wire.h>        // use Wire library for protocol i2c (A4 = SDA & A5 = SCL)
#include <LiquidCrystal_I2C.h>
#include <FastLED.h>
#include <dht.h>

int DHTPIN = 10;
dht DHT; 

//#define DEBUG
//#define WAITONBOOT // If this is enabled the client will be in a "welcome" mode until it receives the first VALID data from the base station
#define LIGHT_THRESHOLD 400

/* Data handling */
char t_in[10], t_out[10];

char Command_PC[64], Command_Serial1[64], Command[64];
static int Count_PC = 0;
static int Count_Serial1 = 0;
static bool Complete_PC = false;
static bool Complete_Serial1 = false;

unsigned int fc = 0;

/* Serial1 Radio 
We don't need to use software serial. The Pro micro has a second serial Serial1.().
*/
//#include <SoftwareSerial.h>
//SoftwareSerial Serial1(A5, A4); // HC-12 TX Pin, HC-12 RX Pin

/* 7Segment: Ledcontrol is used to drive the MAX7212. */
#include "LedControl.h"

                      //DIN,CLK,CS, Number of devices, We have only a single MAX72XX.
LedControl lc=LedControl(7,8,9,1);

/* LCD Screen Settings */
LiquidCrystal_I2C lcd(0x27,20,4);  // set the LCD address to 0x27 for a 16 chars and 2 line display


/* Led Rings
 *  Red and green are switched by this library <= FAIL :***
 *  R G B
 *  G R B
 *  However all you need to do to adjust it is just to initialize the leds in the setup section as GRB.
 */
#define UPDATES_PER_SECOND 100
#define BRIGHTNESS  75
#define NUM_LEDS 41  //  Has to be +1 higher than highest value
CRGB leds[NUM_LEDS]; //  TEMPERATURE RING

#define NUM_LEDS2 17 //  Has to be +1 higher than highest value
CRGB leds2[NUM_LEDS2];// WIND SPEED RING

#define NUM_LEDS3 17 //  Has to be +1 higher than highest value
CRGB leds3[NUM_LEDS3];// WIND DIRECTION RING

// Pins for led rings
#define ring_windspeed   15
#define ring_temperature 14
#define ring_windir      16

CRGBPalette16 currentPalette;
TBlendType    currentBlending;

bool ledfunlock=false;


int ON=HIGH;
int OFF=LOW;
int LON=LOW;
int LOFF=HIGH;

// Hold timers
bool on_hold=false;
uint32_t curr_time;
uint32_t last_event;
uint32_t hold_time=120000; // 120 seconds - initial bootup hold

// Weather telemetry
float temp_in  = 0.00;
float hum_in   = 0.00;

float temp_out = 0.00; 
float hum_out  = 0.00;
float light    = 0.00;
float light_avg= 0.00;
float rain     = 0.00;
float wind     = 0.00;

unsigned int pressure = 0;
unsigned int windiri = 0; // 0 - North
char windc[5]; 

float first_data=false;

char Msg[150];
char MsgReceived[150]; 


/* Want to have custom fun characters?! x_X 
 * https://maxpromer.github.io/LCD-Character-Creator/
 */

byte ico_cyc[8] = {
  B11111,
  B01110,
  B00110,
  B00010,
  B00100,
  B01000,
  B01000,
  B00100
};

byte ico_windir[8] = {
  B00000,
  B00100,
  B01010,
  B10001,
  B01010,
  B00100,
  B00000,
  B00000
};

/*
byte ico_tmp[8] =
{
  B00100,
  B01010,
  B01010,
  B01110,
  B01110,
  B11111,
  B11111,
  B01110
};
*/
byte ico_hum[8] =
{
  B00100,
  B00100,
  B01010,
  B01010,
  B10001,
  B10001,
  B10001,
  B01110,
};

byte ico_sol[8] =
{
  0B00000,
  0B00000,
  0B10101,
  0B01110,
  0B11111,
  0B01110,
  0B10101,
  0B00000,
}; 

byte ico_moon[8] = {
  B00000,
  B00011,
  B01110,
  B11100,
  B11000,
  B11100,
  B01110,
  B00011
};

byte ico_smk[8] = {
  B00010,
  B00000,
  B00100,
  B01110,
  B01110,
  B11111,
  B11111,
  B01110
};

// Searches a string for a pattern character
// returns 0 - if pattern not found
// returns x - the number of occasions the pattern is found
int StrContains(char *str, char *pattern)
{
    char index = 0;
    char len;
    unsigned int occurrence = 0;
    
    len = strlen(str);
    
    if (strlen(pattern) > len) {
        return 0;
    }
    while (index < len) {
        if (str[index] == pattern[0])
            occurrence++;

        index++;
    }

     return occurrence;
}

void str_replace(char *src, char *oldchars, char *newchars) { // utility string function
  char *p = strstr(src, oldchars);
  char buf[100];
  do {
    if (p) {
      memset(buf, '\0', strlen(buf));
      if (src == p) {
        strcpy(buf, newchars);
        strcat(buf, p + strlen(oldchars));
      } else {
        strncpy(buf, src, strlen(src) - strlen(p));
        strcat(buf, newchars);
        strcat(buf, p + strlen(oldchars));
      }
      memset(src, '\0', strlen(src));
      strcpy(src, buf);
    }
  } while (p && (p = strstr(src, oldchars)));
}

/* Incoming command handling */
void xcomm() {

#ifdef DEBUG
/* COMM FROM PC */   
if (Serial.available()){

    Serial.println("PC: New command, collecting...");
    memset(Command_PC, 0, sizeof(Command_PC));
    while (Serial.available()){
      char character = Serial.read();
      if (character=='\n')
         {
          Complete_PC =true;
          Count_PC = 0;
         } else {
             if (Count_PC < 62) { // Don't overflow the array...!
                 Command_PC[Count_PC++] = character;
                 Command_PC[Count_PC] = 0;
             }
         }
  }
}   
#endif

/* COMM FROM Serial1 */   
if (Serial1.available()){

  #ifdef DEBUG
    Serial.print("Serial1: ");
  #endif
    memset(Command_Serial1, 0, sizeof(Command_Serial1));
    while (Serial1.available()){
      char character = Serial1.read();
      /*
      Serial.print("C:");
      Serial.print(character);
      */
      delay(100); /* DO NOT REMOVE THIS WITH LOW BPS like 1200Bps or expect large amount of broken data */
      if (character=='\n')
         {
          Complete_Serial1 =true;          
          Count_Serial1 = 0;
         } else {
             if (Count_Serial1 < 62) { // Don't overflow the array...!
                 Command_Serial1[Count_Serial1++] = character;
                 Command_Serial1[Count_Serial1] = 0;
             }
         }
  }
}   
  

/* Command_PC is just for debugging, disabled normally */
 if (Complete_PC||Complete_Serial1) {
  
  strcpy(Command, Command_Serial1);
  #ifdef DEBUG
  if (Complete_Serial1)
  {
    strcpy(Command, Command_Serial1);
    Serial.print("Command received Serial1: |"); // For debug we use the | as xmit begin/stop indicator as the remote will send > <
    Serial.print(Command);
    Serial.println("|");
  }
  
  if (Complete_PC)
  {
    strcpy(Command, Command_PC);
    Serial.print("Command received PC (overwriting Serial1): |");
    Serial.print(Command);
    Serial.println("|");
  }
  #endif  

/*  
 * 8 Data points: >temperature,humidity,lightsensitivity,light_avg,pressure,wind,winddirection,rain<
 * From these all is float except the windirection which is an unsigned int between 0-15

   Step 1: look for both of the >markers< neededs to be present 
   Step 2: 8 comma separated values needs to be present
*/
  if (StrContains(Command,">") == 1 && StrContains(Command,"<") == 1 && StrContains(Command,",") == 7) {
    
                    // Step 3: remove markers
                    str_replace(Command, ">", "");
                    str_replace(Command, "<", "");


                    
                    temp_out = atof(strtok(Command, ","));
                    hum_out  = atof(strtok(NULL, ","));
                    light    = atof(strtok(NULL, ","));
                    light_avg= atof(strtok(NULL, ","));
                    pressure = atoi(strtok(NULL, ","));                   
                    wind     = atof(strtok(NULL, ","));
                    windiri  = atoi(strtok(NULL, ","));
                    rain     = atof(strtok(NULL, ","));
                    
                  #ifdef DEBUG
                    Serial.println("Data decoded: ");
                    Serial.print("var1 temp: >");
                    Serial.println(temp_out);                  
                    Serial.print("var2 hum: >");
                    Serial.println(hum_out);                  
                    Serial.print("var3 light: >");
                    Serial.println(light);                  
                    Serial.print("var4 light_avg: >");
                    Serial.println(light_avg);                        
                    Serial.print("var5 pressure: >");
                    Serial.println(pressure);                  
                    Serial.print("var6 wind: >");
                    Serial.println(wind);                  
                    Serial.print("var7 wind direction: >");
                    Serial.println(windiri);                  
                    Serial.print("var8 rain amount: >");
                    Serial.println(rain);                  
                  #endif
                 // fc=0; // Comment this out for regression testing and leave it for days
                  first_data=true;
   } else if (strlen(Command) < 5) {
     fc++;
     // Some absolute junk completely ignore this don't even alert
   } else { // The base station don't accept incoming commands, in case of broken xmit no need to cry about it.   
       fc++;
       ring_alarm();
       #ifdef DEBUG
          Serial.println("CYCLONE_CLIENT_ERR_UNK_DATA");            
       #endif    
   }
                        
                       
       Complete_PC=false;
       Complete_Serial1=false;
 }
 
}
        

void rings_reset()
{
  unsigned int i;
  for (i=0; i<NUM_LEDS-1; i++)
   {
    leds[i] = CRGB::Black;
    FastLED.show();
    }

  for (i=0; i<NUM_LEDS2-1; i++)
   {
    leds2[i] = CRGB::Black;
    leds3[i] = CRGB::Black;
    FastLED.show();
    }

}

void rings_red()
{
  unsigned int i;
  for (i=0; i<NUM_LEDS-1; i++)
   {
    leds[i] = CRGB::DarkRed;
    FastLED.show();
    }

  for (i=0; i<NUM_LEDS2-1; i++)
   {
    leds2[i] = CRGB::DarkRed;
    leds3[i] = CRGB::DarkRed;
    FastLED.show();
    }

}

void ring_alarm() {
  for (unsigned int i=0; i<3;i++) {
   rings_red();
   delay(1000);
   rings_reset();
   delay(1000);
  }
}

/* This dial obviously has to scale. I looked up my record wind in my area pick something under eg 90kmh as top.
 * This is a linear conversion with the formula:
 * new_value = ( (old_value - old_min) / (old_max - old_min) ) * (new_max - new_min) + new_min
 * Also in this case we don't give a crap about absolute 0 wind, the function will just return.
*/

void ledring_color_windspeed(float i)
{  
  //Max
  if(i<1) return;
  if(i >90) i = 90;

  unsigned int j=0;
  float wind_scaled=(i/90)*16;  // Has to float
  
  for (j=0; j<=wind_scaled; j++) { // This is cheating since we only have 0-15 addressable leds but we calculate with the "led amount" 16 so needs adjustment

   if (wind_scaled>=0 && wind_scaled<=4)
    leds2[j] = CRGB::Teal;
   else if (wind_scaled>=5 && wind_scaled<=8)
    leds2[j] = 0xC728D7; // Purple
   else if (wind_scaled>=9 && wind_scaled<=12)
    leds2[j] = 0xFF8000; // Orange
   else if (wind_scaled>=13 && wind_scaled<=16)
    leds2[j] = CRGB::DarkRed;

   FastLED.show();
  }

   // Starts from the next led (+1) from the last ON
   for (; j<=15; j++) // Black out the other half of the ring
   {
    leds2[j] = CRGB::Black;
    FastLED.show();
    }

}

/* Wind Direction Ring 
 * Lights up one single led, blacks out all others
 * This ring goes from 0-15 (this is the range it will get from the base)
 * any other input and it returns.
*/
void ledring_color_windir(unsigned int windy)
{
  //Max
  if(windy<0 || windy >15) return;
  
   unsigned int i;      
   leds3[windy] = 0xEC00FF;
   FastLED.show();
      
   for (i=0; i<16; i++) {
       if(i != windy)
       {
          leds3[i] = CRGB::Black;
          FastLED.show();
       }
      }
}

/* 
 This uses multiple color palettes depending on the temperature 
 and clockwise/counter clockwise direction based on the temperature
 is negative or positive. 

 This code instead of relying on a full ring reset will automatically
 black out the rest of the ring for faster execution.

*/

void ledring_color_temperature(float temp)
{
  int i;
  float negatemp;
  
 // This is the max
 if(temp>40)  temp =  40;
 if(temp<-40) temp = -40;

/*
 Serial.println("RING TEMP: ");
 Serial.println(temp);
*/
 if(temp>=0) 
 {
   for (i=0; i<=temp; i++) // Fill in the positive side of the ring
   {
     if (temp==0)
      leds[i] = CRGB::White; 
     else if(temp>0 && temp<15)
      leds[i] = CRGB::DarkBlue; 
     else if(temp>=15 && temp<25)
      leds[i] = CRGB::DarkGreen;
     else if(temp>=25 && temp<=40) 
      leds[i] = CRGB::DarkRed;

      FastLED.show();
    }

   for (i=temp+1; i<=40; i++) // Black out the other half of the ring
   { 
    leds[i] = CRGB::Black;
    FastLED.show();
    }
   
 } else {
  negatemp=40+temp; // eg 40+(-8) = 32
  

  // This goes from 40 back to x
   for (i=40; i>=negatemp; i--) 
   {
     
     if(negatemp<20)
      leds[i] = CRGB::Teal;
     else     
      leds[i] = CRGB::DarkBlue;
     
     FastLED.show();
     
    }  
    
   // This goes from x down to 0
   for (i=40+temp-1; i>=0; i--) // Black out the other half of the ring
   {
    leds[i] = CRGB::Black;
    FastLED.show();
    } 

    
 }
  
}

/********************************************** Led Game **********************************************/

void SetupTotallyRandomPalette()
{
    for( int i = 0; i < 16; i++) {
        currentPalette[i] = CHSV( random8(), 255, random8());
    }
}
void SetupBlackAndWhiteStripedPalette()
{
    // 'black out' all 16 palette entries...
    fill_solid( currentPalette, 16, CRGB::Black);
    // and set every fourth one to white.
    currentPalette[0] = CRGB::White;
    currentPalette[4] = CRGB::White;
    currentPalette[8] = CRGB::White;
    currentPalette[12] = CRGB::White;
    
}
void SetupPurpleAndGreenPalette()
{
    CRGB purple = CHSV( HUE_PURPLE, 255, 255);
    CRGB green  = CHSV( HUE_GREEN, 255, 255);
    CRGB black  = CRGB::Black;
    
    currentPalette = CRGBPalette16(
                                   green,  green,  black,  black,
                                   purple, purple, black,  black,
                                   green,  green,  black,  black,
                                   purple, purple, black,  black );
}

const TProgmemPalette16 myRedWhiteBluePalette_p PROGMEM =
{
    CRGB::Red,
    CRGB::Gray, // 'white' is too bright compared to red and blue
    CRGB::Blue,
    CRGB::Black,
    
    CRGB::Red,
    CRGB::Gray,
    CRGB::Blue,
    CRGB::Black,
    
    CRGB::Red,
    CRGB::Red,
    CRGB::Gray,
    CRGB::Gray,
    CRGB::Blue,
    CRGB::Blue,
    CRGB::Black,
    CRGB::Black
};

void ChangePalettePeriodically()
{
    uint8_t secondHand = (millis() / 1000) % 60;
    static uint8_t lastSecond = 99;
    
    if( lastSecond != secondHand) {
        lastSecond = secondHand;
        if( secondHand ==  0)  { currentPalette = RainbowColors_p;         currentBlending = LINEARBLEND; }
        if( secondHand == 10)  { currentPalette = RainbowStripeColors_p;   currentBlending = NOBLEND;  }
        if( secondHand == 15)  { currentPalette = RainbowStripeColors_p;   currentBlending = LINEARBLEND; }
        if( secondHand == 20)  { SetupPurpleAndGreenPalette();             currentBlending = LINEARBLEND; }
        if( secondHand == 25)  { SetupTotallyRandomPalette();              currentBlending = LINEARBLEND; }
        if( secondHand == 30)  { SetupBlackAndWhiteStripedPalette();       currentBlending = NOBLEND; }
        if( secondHand == 35)  { SetupBlackAndWhiteStripedPalette();       currentBlending = LINEARBLEND; }
        if( secondHand == 40)  { currentPalette = CloudColors_p;           currentBlending = LINEARBLEND; }
        if( secondHand == 45)  { currentPalette = PartyColors_p;           currentBlending = LINEARBLEND; }
        if( secondHand == 50)  { currentPalette = myRedWhiteBluePalette_p; currentBlending = NOBLEND;  }
        if( secondHand == 55)  { currentPalette = myRedWhiteBluePalette_p; currentBlending = LINEARBLEND; }
    }
}

void FillLEDsFromPaletteColors( uint8_t colorIndex)
{
    uint8_t brightness = 255;
    
    for( int i = 0; i < NUM_LEDS; i++) {
        leds[i] = ColorFromPalette( currentPalette, colorIndex, brightness, currentBlending);
        colorIndex += 3;
    }

    for( int z = 0; z < NUM_LEDS2; z++) {
        leds2[z] = ColorFromPalette( currentPalette, colorIndex, brightness, currentBlending);
        leds3[z] = ColorFromPalette( currentPalette, colorIndex, brightness, currentBlending);
        colorIndex += 3;
    }

}

/*
LED Fun is used at two places:

1, During the initialization of the station (until it gets any data from the base, the ledgame will be played endlessly)
2, At dawn when the LightAVG reaches the critical treshold

The ledgame is moved to the very end of each loop and lasts for 30 seconds so if the station received weather telemetry
it will be displayed first before this plays and won't interfere with the next incoming transmission if you use 1 minute sampling. 
*/

void led_fun()
{
  int i;

  // 2400 iterations will give an approximate runtime of 30 seconds
  for (i=0; i<2400; i++)
   {
    ChangePalettePeriodically();
    static uint8_t startIndex = 0;
    startIndex = startIndex + 1; /* motion speed */
    
    FillLEDsFromPaletteColors(startIndex);   
    FastLED.show();
    FastLED.delay(1000 / UPDATES_PER_SECOND);
    }
    rings_reset();
}
/********************************************** Led Game **********************************************/

// Error x_X
void display_err() {
    lc.clearDisplay(0);  
    lc.setChar(0,6,'E',false);
    lc.setChar(0,5,'E',false);
    lc.setChar(0,4,'E',false);

    lc.setChar(0,2,'E',false);
    lc.setChar(0,1,'E',false);
    lc.setChar(0,0,'E',false);   
}

/********************************************** 7 SEGMENT DUAL TEMPERATURE **********************************************/

/*
  This function expect 2 decimal padded floats (in char format) any bigger number will just makes it return. Example valid inputs:
  >12.52<
  >9.24<
  >-11.75<
  >-8.74<
 
   Some pain: scanf will not read the float numbers and printf will not print them either so we don't even bother with 
   numbers in this function just process char arrays. Anyway the 7segment has no "real understanding" of what a floating point number
   is or how to even shift it in our case. Therefore this custom handler works with chars directly. This function will check if the 
   incoming string does start with - minus if it is then it displays the negative if not it will pad the display, keeping that digit off.  
*/ 
void display_temperature(char *temp_outside, char *temp_inside) {

   if(strlen(temp_outside) < 4 || strlen(temp_outside) > 6)
    {
          display_err();
          delay(1000);
          lc.clearDisplay(0);
          #ifdef DEBUG
           Serial.print("ERROR in display input data:");
           Serial.println(temp_outside);
          #endif      
          return;
    }
   
    // Last digit at the right
    if (temp_outside[0] == '-') // NEGATIVE TEMPERATURE
     {
      lc.setChar(0,7,'-',false); // Shift them    
      
       if (temp_outside[2] == '.')  // This is a single digit temperature, ignore the dot print the next number + dot and the final number eg: 9.1
        {
         lc.setChar(0,6,temp_outside[1],true);      
         lc.setChar(0,5,temp_outside[3],false);
         lc.setChar(0,4,temp_outside[4],false);
        }
       else if (temp_outside[3] == '.') // This is a multi digit temperature display the number display the dot after the second digit eg: 19.1
        {
         lc.setChar(0,6,temp_outside[1],false);      
         lc.setChar(0,5,temp_outside[2],true);        
         lc.setChar(0,4,temp_outside[4],false);
        }
       else  // The number must be float, if not add display error
        {
          display_err();
          #ifdef DEBUG
           Serial.print("ERROR in display input data:");
           Serial.println(temp_outside);
          #endif
        }
     } else { // POSITIVE TEMPERATURE
       if (temp_outside[1] == '.')  // This is a single digit temperature, ignore the dot print the next number + dot and the final number eg: 9.12
        {
         lc.setChar(0,7,temp_outside[0],true);      
         lc.setChar(0,6,temp_outside[2],false);      
         lc.setChar(0,5,temp_outside[3],false);
         // This is the only case when we don't have a character here due to the script takes %.2f format, whatever is in array[4] is junk
         lc.setChar(0,4,' ',false);                
        }
       else if (temp_outside[2] == '.') // This is a multi digit temperature display the number display the dot after the second digit eg: 19.12
        {
         lc.setChar(0,7,temp_outside[0],false);                
         lc.setChar(0,6,temp_outside[1],true);      
         lc.setChar(0,5,temp_outside[3],false);        
         lc.setChar(0,4,temp_outside[4],false);
        }
       else  // The number must be float, if not add display error
        {
          display_err();
          #ifdef DEBUG
           Serial.print("ERROR in display input data:");
           Serial.println(temp_outside);
          #endif
        }      
     }
     
    // First digit from the left
    if (temp_inside[0] == '-') // NEGATIVE TEMPERATURE
     {
      lc.setChar(0,3,'-',false); // Shift them    
      
       if (temp_inside[2] == '.')  // This is a single digit temperature, ignore the dot print the next number + dot and the final number eg: 9.1
        {
         lc.setChar(0,2,temp_inside[1],true);      
         lc.setChar(0,1,temp_inside[3],false);
         lc.setChar(0,0,temp_inside[4],false);
        }
       else if (temp_inside[3] == '.') // This is a multi digit temperature display the number display the dot after the second digit eg: 19.1
        {
         lc.setChar(0,2,temp_inside[1],false);      
         lc.setChar(0,1,temp_inside[2],true);        
         lc.setChar(0,0,temp_inside[4],false);
        }
       else  // The number must be float, if not add display error
        {
          display_err();
          #ifdef DEBUG
           Serial.print("ERROR in display input data:");
           Serial.println(temp_inside);
          #endif
        }
     } else { // POSITIVE TEMPERATURE
       if (temp_inside[1] == '.')  // This is a single digit temperature, ignore the dot print the next number + dot and the final number eg: 9.12
        {
         lc.setChar(0,3,temp_inside[0],true);      
         lc.setChar(0,2,temp_inside[2],false);      
         lc.setChar(0,1,temp_inside[3],false);
         // This is the only case when we don't have a character here due to the script takes %.2f format, whatever is in array[4] is junk
         lc.setChar(0,0,' ',false);                
        }
       else if (temp_inside[2] == '.') // This is a multi digit temperature display the number display the dot after the second digit eg: 19.12
        {
         lc.setChar(0,3,temp_inside[0],false);                
         lc.setChar(0,2,temp_inside[1],true);      
         lc.setChar(0,1,temp_inside[3],false);        
         lc.setChar(0,0,temp_inside[4],false);
        }
       else  // The number must be float, if not add display error
        {
          display_err();
          #ifdef DEBUG
           Serial.print("ERROR in display input data:");
           Serial.println(temp_inside);
          #endif
        }      
     }

    
}

/* Decodes wind direction from integers.
   The reason why this is implemented the way it is because if you mount the station facing different dirrection eg: North is East.
   All you need to do is shift the incoming number with +3.
*/
void windir_decode(unsigned int wind) {  

  if(wind>15) return;

  switch (wind) {
  case 0:    
    strcpy(windc, "N");
    break;
  case 1:
    strcpy(windc, "NE1");
    break;
  case 2:
    strcpy(windc, "NE2");
    break;
  case 3:
    strcpy(windc, "NE3");
    break;
  case 4:
    strcpy(windc, "E");
    break;        
  case 5:
    strcpy(windc, "SE1");
    break;   
  case 6:
    strcpy(windc, "SE2");
    break;   
  case 7:
    strcpy(windc, "SE3");
    break;
  case 8:
    strcpy(windc, "S");
    break;
  case 9:
    strcpy(windc, "SW1");
    break;
  case 10:
    strcpy(windc, "SW2");
    break;
  case 11:
    strcpy(windc, "SW3");
    break;
  case 12:
    strcpy(windc, "W");
    break;
  case 13:
    strcpy(windc, "NW1");
    break;
  case 14:
    strcpy(windc, "NW2");
    break;
  case 15:
    strcpy(windc, "NW3");
    break;    
 } 
}

void setup() {

/* Initialize the LCD */
 lcd.init();
 lcd.backlight();
 
// lcd.createChar(1, ico_tmp);
 lcd.createChar(2, ico_hum);
 lcd.createChar(3, ico_sol);
 lcd.createChar(4, ico_smk);
 lcd.createChar(5, ico_cyc);
 lcd.createChar(6, ico_windir);
 lcd.createChar(7, ico_moon);
 lcd.clear();


/* Initialize the MAX72XX
   The MAX72XX is in power-saving mode on startup,
   we have to do a wakeup call
*/
 lc.shutdown(0,false);
/* Set the brightness to a medium values */
 lc.setIntensity(0,8);
/* and clear the display */
 lc.clearDisplay(0);

/* Initialize the LED rings */

  FastLED.addLeds<WS2812B, ring_temperature, GRB>(leds,  NUM_LEDS);
  FastLED.addLeds<WS2812B, ring_windspeed, GRB>  (leds2, NUM_LEDS2);
  FastLED.addLeds<WS2812B, ring_windir, GRB>     (leds3, NUM_LEDS3);
  FastLED.setBrightness(BRIGHTNESS); 
  rings_reset();

#ifdef DEBUG
 Serial.begin(9600);  // USB
// while (!Serial);
// Serial.println("CyClone Client ... [ STARTING ]");
 Serial.println("LCD Intro");
#endif

/* Serial1 Radio */
 Serial1.begin(1200);    // Serial1
 
/* Add initial bootup wait here led games + lcd intro text */
  
/* Print LCD intro */
  lcd.setCursor(0, 0);  
  lcd.write(5);
  lcd.print(" CyClone station  ");
  lcd.write(5);
  lcd.setCursor(0, 1);  
  lcd.print("  ");
  lcd.write(5);
  lcd.print(" by NLD 2022 ");  
  lcd.write(5);
  lcd.print(" ");
  lcd.setCursor(0, 2);  
  lcd.print("Waiting for initial ");  
  lcd.setCursor(0, 3);  
  lcd.print("data from base ...");  

#ifdef WAITONBOOT
  curr_time = millis();
  unsigned int l=0;
  
  while(!first_data) {
     
     xcomm(); 
     delay(1000);
    // Limit events
    if (on_hold && curr_time - last_event >= hold_time) {
        on_hold = false;
    }

    if (!on_hold)
    {
      led_fun(); 
      last_event = curr_time;
      on_hold = true;
    }
  }  
#endif
  delay(5000);
  lcd.clear();
}  

void loop() {

  xcomm(); // Collect data on radio
  curr_time = millis();

  DHT.read22(DHTPIN);        // DHT22 (Inside sensor)
  temp_in = DHT.temperature; // Gets the values of the temperature
  hum_in = DHT.humidity;     // Gets the values of the humidity
  
  dtostrf(temp_out, 2, 2, t_out);
  dtostrf(temp_in,  2, 2, t_in);
  display_temperature(t_out,t_in); // 7-Segment

  windir_decode(windiri);
  
  ledring_color_temperature(temp_out);
  ledring_color_windspeed(wind);
  ledring_color_windir(windiri);         
    
/* Display on the LCD  */

// Row1: Humidity

lcd.setCursor(0, 0);  

lcd.write(2);
lcd.print(" ");
lcd.print(hum_out);
lcd.print("%  "); // Extra padding for junk

lcd.setCursor(11, 0);  // This is useful if the data length varies and  you want to start the next from a fix position

lcd.write(2);
lcd.print(" ");
lcd.print(hum_in);
lcd.print("%     "); // Extra padding for junk

// Row 2: light + fail counter

lcd.setCursor(0, 1);  
if(light_avg>LIGHT_THRESHOLD)
 lcd.write(3); // Sun
else 
 lcd.write(7); // Moon
lcd.print(" ");
lcd.print(light);
lcd.print(" ");
if(light_avg>LIGHT_THRESHOLD)
 lcd.write(3); // Sun
else
 lcd.write(7); // Moon
lcd.print(" lux F "); // Extra padding for junk
lcd.print(fc);
lcd.print("    ");

// Row 3: Wind
lcd.setCursor(0, 2); 
lcd.write(5);
lcd.print(" "); 
lcd.print(wind);
lcd.print(" km/h ");  
lcd.write(6);
lcd.print(" ");
lcd.print(windc);
lcd.print(" ");
lcd.write(6);

// Row4: Rain + Barometric pressure
lcd.setCursor(0, 3);  
lcd.write(4);
lcd.print(" ");  
lcd.print(rain);
lcd.print(" mL/h ");
lcd.print("       "); // Extra padding for junk
lcd.setCursor(13, 3);  // This is useful if the data length varies and  you want to start the next from a fix position
lcd.print(pressure);

if(pressure<1000)  // Little trick to get rid of the last possible junk character however this needs to be controlled +2 and it overruns
 lcd.print("hPa "); 
else
 lcd.print("hPa"); 


// Led game when dark sets in
if(light_avg>LIGHT_THRESHOLD) {
 ledfunlock=false;
} else {
 if (!ledfunlock) {
   led_fun();
   ledfunlock=true; // Lock, only sun can unlock it
  }
}

//This delay is necessary otherwise the LCD would be flickering like crazy from the constant changes
delay(1000);

}
